<?php

namespace FM\Service;

use FM\Factory\Fm00010000ParametrizacaoGeralFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm00010000ParametrizacaoGeralService
{

    protected $ParametrizacaoGeralFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(Fm00010000ParametrizacaoGeralFactory $ParametrizacaoGeralFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->ParametrizacaoGeralFactory   = $ParametrizacaoGeralFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getParametrizacoes()
    {
        return $this->ParametrizacaoGeralFactory->getParametrizacoes();
    }

    public function salvarParametrizacoes($params)
    {
        $this->connection->beginTransaction();
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
            
            $retorno = $this->ParametrizacaoGeralFactory->getParametrizacoes();
            $nrParam = isset($retorno[0]['NRPARAMGERAL']) ? $retorno[0]['NRPARAMGERAL'] : 'N';

            if (isset($params['TAREFA'])) {
                $this->salvarParametrizacaoTarefa($params, $nrParam);
            } else {
                if ($params['PERGEOLOCALIZACAO'] == "N") {
                    $params['INICIARTARGEOLOCALIZACAO'] = "N";
                }

                $arrayModificado = array();
                foreach ($params['LISTNOTIFNAOCONFORM'] as $valor) {
                    array_push($arrayModificado, $valor);
                }
                $params['LISTNOTIFNAOCONFORM'] = implode('_', $arrayModificado);

                if ($nrParam != 'N') {
                    $param  = array(
                        "CDOPERULTATU"            => $CDOPERADOR,
                        "IDUTILIZAGEOLOCAL"       => $params['PERGEOLOCALIZACAO'],
                        "IDUTILIZAGEOLOCALINITAR" => $params['INICIARTARGEOLOCALIZACAO'],
                        "IDHABILITAOFFLINE"       => $params['USAOFFLINE'],
                        "DTINICARREGAOFFLINE"     => isset($params["DATARANGE"][0]) ? $params["DATARANGE"][0] : null,
                        "DTFIMCARREGAOFFLINE"     => isset($params["DATARANGE"][1]) ? $params["DATARANGE"][1] : null,
                        "QTDIASCARREGAOFFLINE"    => ($params["XDIAS"] != "") ? $params["XDIAS"] : null,
                        "NRPARAMGERAL"            => $nrParam,
                        "IDTIPOCARREGAOFFLINE"    => $params["TIPOCARREGAMENTO"],
                        "IDATIVO"                 => 'S',
                        'NRORGULTATU'             => $NRORG,
                        "IDNOTIFNAOCONFORM"       => $params['IDNOTIFNAOCONFORM'],
                        "LISTNOTIFNAOCONFORM"     => $params['LISTNOTIFNAOCONFORM'],
                        "EMAILSNOTIFNAOCONFROM"   => $params['EMAILSNOTIFNAOCONFROM'],
                    );

                    $this->ParametrizacaoGeralFactory->updateParametrizacao($param);
                } else {
                    $param  = array(
                        "NRPARAMGERAL"            => $this->newCode->getCodeByTableName("FCM_PARAMGERAL", $NRORG, 1, 10, true),
                        "NRORG"                   => $NRORG,
                        "IDATIVO"                 => 'S',
                        "CDOPERINCLUSAO"          => $CDOPERADOR,
                        "NRORGINCLUSAO"           => $NRORG,
                        "IDUTILIZAGEOLOCAL"       => $params['PERGEOLOCALIZACAO'],
                        "IDUTILIZAGEOLOCALINITAR" => $params['INICIARTARGEOLOCALIZACAO'],
                        "IDHABILITAOFFLINE"       => $params['USAOFFLINE'],
                        "IDTIPOCARREGAOFFLINE"    => $params["TIPOCARREGAMENTO"],
                        "QTDIASCARREGAOFFLINE"    => ($params["XDIAS"] != "") ? $params["XDIAS"] : null,
                        "DTINICARREGAOFFLINE"     => isset($params["DATARANGE"][0]) ? $params["DATARANGE"][0] : null,
                        "DTFIMCARREGAOFFLINE"     => isset($params["DATARANGE"][1]) ? $params["DATARANGE"][1] : null,
                        "IDNOTIFNAOCONFORM"       => $params['IDNOTIFNAOCONFORM'],
                        "LISTNOTIFNAOCONFORM"     => $params['LISTNOTIFNAOCONFORM'],
                        "EMAILSNOTIFNAOCONFROM"   => $params['EMAILSNOTIFNAOCONFROM'],
                    );
                    
                    $this->ParametrizacaoGeralFactory->inserirParametrizacao($param);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarParametrizacaoTarefa($parametros, $nrParam)
    {
        $this->connection->beginTransaction();

        
        try {
            $CDOPERADOR = $this->Environment->getCdOperador();
            $NRORG = $this->Environment->getNrOrg();
        
            if ($nrParam != 'N') {
                $param  = array(
                    "CDOPERULTATU"            => $CDOPERADOR,
                    "NRORGULTATU"             => $NRORG,
                    "IDVALIDACRIATAREFA"      => $parametros['CONFLITO'],
                    "IDUTILIZAHABILIDADE"     => $parametros['HABILIDADEOPERADOR'],
                    "IDVALIDACONFLITOURG"     => $parametros['IDVALIDACONFLITOURG'],
                    "NRPARAMGERAL"            => $nrParam,
                );

                 $this->ParametrizacaoGeralFactory->updateParametrizacaoTarefa($param);
            } else {
                $param  = array(
                    "NRPARAMGERAL"            => $this->newCode->getCodeByTableName("FCM_PARAMGERAL", $NRORG, 1, 10, true),
                    "NRORG"                   => $NRORG,
                    "CDOPERINCLUSAO"          => $CDOPERADOR,
                    "NRORGINCLUSAO"           => $NRORG,
                    "IDVALIDACRIATAREFA"      => $parametros['CONFLITO'],
                    "IDUTILIZAHABILIDADE"     => $parametros['HABILIDADEOPERADOR'],
                    "IDVALIDACONFLITOURG"     => $parametros['IDVALIDACONFLITOURG'],
                );
                
                $this->ParametrizacaoGeralFactory->inserirParametrizacaoTarefa($param);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getMascara()
    {
        return $this->ParametrizacaoGeralFactory->getMascara();
    }

    // Integração com SM
    public function salvarRelacionamento($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
            $contador = array();

            foreach ($parametros['ATIVIDADE'] as $item) {
                $retorno = $this->ParametrizacaoGeralFactory->validarItem(array(
                    "NRATIVIDADE"       => $item['NRATIVIDADE'],
                    "CDSERVORCA"        => $parametros['CDSERVORCA'],
                    "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV']
                ));

                if ($retorno == 0) {
                    $params = array(
                        "NRRELSERVITEMTAR"  => $this->newCode->getCodeByTableName("FCM_PARAMGERAL", $NRORG, 1, 10, true),
                        "NRATIVIDADE"       => $item['NRATIVIDADE'],
                        "CDSERVORCA"        => $parametros['CDSERVORCA'],
                        "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV'],
                        "NRORG"             => $NRORG,
                        "NRORGINCLUSAO"     => $NRORG,
                        "CDOPERINCLUSAO"    => $cdOperador
                    );

                    $this->ParametrizacaoGeralFactory->inserirRelacionamento($params);
                } else {
                    array_push($contador, $item['NMATIVIDADE']);
                }
            }
            $this->connection->commit();
            return $contador;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getRelacionamentoServicoTipoMedida()
    {
        return $this->ParametrizacaoGeralFactory->getRelacionamentoServicoTipoMedida();
    }

    public function getItensRelacionamentoServicoTipoMedida($parametros)
    {
        return $this->ParametrizacaoGeralFactory->getItensRelacionamentoServicoTipoMedida(array(
            "CDSERVORCA"        => $parametros['CDSERVORCA'],
            "CDTPMEDIDASERV"    => $parametros['CDTPMEDIDASERV'],
        ));
    }

    public function inativarServicos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ParametrizacaoGeralFactory->inativarServicos(array(
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $NRORG,
                "CDSERVORCA"        => $parametros['NRS'][0]['CDSERVORCA'],
                "CDTPMEDIDASERV"    => $parametros['NRS'][0]['CDTPMEDIDASERV']
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Funções da Tab de Tarefas
    public function getPrioridades()
    {
        return $this->ParametrizacaoGeralFactory->getPrioridades();
    }

    public function insertPrioridade($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros['ROW'] as $row) {
                if ($row['NRPRIORIDADETAREFA'] === 0) {
                    $this->ParametrizacaoGeralFactory->insertPrioridade(array(
                        "NRPRIORIDADETAREFA"    => $this->newCode->getCodeByTableName("FCM_PRIORIDADETAREFA", $NRORG, 1, 10, true),
                        "NMPRIORIDADETAREFA"    => $row['NMPRIORIDADETAREFA'],
                        "IDPRIORIDADE"          => $row['IDPRIORIDADE'],
                        "ATRASOPERMITIDO"       => (isset($row['ATRASOPERMITIDO'])) ? $row['ATRASOPERMITIDO'] : null,
                        "IDATIVO"               => $row['IDATIVO'],
                        "NRORG"                 => $NRORG,
                        "CDOPERINCLUSAO"        => $cdOperador,
                        "NRORGINCLUSAO"         => $NRORG,
                    ));
                } else {
                    $this->ParametrizacaoGeralFactory->updatePrioridade(array(
                        "NRPRIORIDADETAREFA"    => $row['NRPRIORIDADETAREFA'],
                        "NMPRIORIDADETAREFA"    => $row['NMPRIORIDADETAREFA'],
                        "IDPRIORIDADE"          => $row['IDPRIORIDADE'],
                        "ATRASOPERMITIDO"       => $row['ATRASOPERMITIDO'],
                        "IDATIVO"               => $row['IDATIVO'],
                        "CDOPERULTATU"          => $cdOperador,
                        "NRORGULTATU"           => $NRORG,
                    ));
                }
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
